/*
 * System V STREAMS TCP - Release 3.0 
 *
 * Copyright 1987, 1988, 1989 Lachman Associates, Incorporated (LAI) 
 * All Rights Reserved. 
 *
 * The copyright above and this notice must be preserved in all copies of this
 * source code.  The copyright above does not evidence any actual or intended
 * publication of this source code. 
 *
 * This is unpublished proprietary trade secret source code of Lachman
 * Associates.  This source code may not be copied, disclosed, distributed,
 * demonstrated or licensed except as expressly authorized by Lachman
 * Associates. 
 *
 * System V STREAMS TCP was jointly developed by Lachman Associates and
 * Convergent Technologies. 
 */
/*
 * Copyright (c) 1985 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Berkeley.  The name of the
 * University may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/* @(#)ip.h	3.3 LAI System V.3 STREAMS TCP/IP  source        */
/* Convergent Technologies - System V - Oct 1985	 */
#ifndef ip_h
#define ip_h
#ident	"@(#)ip.h	6.6 :/source/uts/common/netinet/s.ip.h 8/15/86 14:29:14"

/* ip.h	6.1	83/07/29	 */

/*
 * Definitions for internet protocol version 4. Per RFC 791, September 1981. 
 */
#define	IPVERSION	4

/*
 * Structure of an internet header, naked of options. 
 *
 * We declare ip_len and ip_off to be short, rather than u_short pragmatically
 * since otherwise unsigned comparisons can result against negative integers
 * quite easily, and fail in subtle ways. 
 */
struct ip {
#if defined(vax) || defined(i386)
	u_char          ip_hl:4,/* header length */
	                ip_v:4;	/* version */
#else
	u_char          ip_v:4,	/* version */
	                ip_hl:4;/* header length */
#endif
	u_char          ip_tos;	/* type of service */
	short           ip_len;	/* total length */
	u_short         ip_id;	/* identification */
	short           ip_off;	/* fragment offset field */
#define	IP_DF 0x4000		/* dont fragment flag */
#define	IP_MF 0x2000		/* more fragments flag */
	u_char          ip_ttl;	/* time to live */
	u_char          ip_p;	/* protocol */
	u_short         ip_sum;	/* checksum */
	struct in_addr  ip_src, ip_dst;	/* source and dest address */
};

#define IP_MAXPACKET 65535		/* maximum packet size */
/*
 * Definitions for options. 
 */
#define	IPOPT_COPIED(o)		((o)&0x80)
#define	IPOPT_CLASS(o)		((o)&0x60)
#define	IPOPT_NUMBER(o)		((o)&0x1f)

#define	IPOPT_CONTROL		0x00
#define	IPOPT_RESERVED1		0x20
#define	IPOPT_DEBMEAS		0x40
#define	IPOPT_RESERVED2		0x60

#define	IPOPT_EOL		0	/* end of option list */
#define	IPOPT_NOP		1	/* no operation */

#define	IPOPT_RR		7	/* record packet route */
#define	IPOPT_TS		68	/* timestamp */
#define	IPOPT_SECURITY		130	/* provide s,c,h,tcc */
#define	IPOPT_LSRR		131	/* loose source route */
#define	IPOPT_SATID		136	/* satnet id */
#define	IPOPT_SSRR		137	/* strict source route */

/*
 * Offsets to fields in options other than EOL and NOP. 
 */
#define	IPOPT_OPTVAL		0	/* option ID */
#define	IPOPT_OLEN		1	/* option length */
#define IPOPT_OFFSET		2	/* offset within option */
#define	IPOPT_MINOFF		4	/* min value of above */

/*
 * Time stamp option structure. 
 */
struct ip_timestamp {
	u_char          ipt_code;	/* IPOPT_TS */
	u_char          ipt_len;/* size of structure (variable) */
	u_char          ipt_ptr;/* index of current entry */
#if defined(vax) || defined(i386)
	u_char          ipt_flg:4,	/* flags, see below */
	                ipt_oflw:4;	/* overflow counter */
#else
	u_char          ipt_oflw:4,	/* overflow counter */
	                ipt_flg:4;	/* flags, see below */
#endif
	union ipt_timestamp {
		n_long          ipt_time[1];
		struct ipt_ta {
			struct in_addr  ipt_addr;
			n_long          ipt_time;
		}               ipt_ta[1];
	} ipt_timestamp;
};

/* flag bits for ipt_flg */
#define	IPOPT_TS_TSONLY		0	/* timestamps only */
#define	IPOPT_TS_TSANDADDR	1	/* timestamps and addresses */
#define	IPOPT_TS_PRESPEC	2	/* specified modules only */

/* bits for security (not byte swapped) */
#define	IPOPT_SECUR_UNCLASS	0x0000
#define	IPOPT_SECUR_CONFID	0xf135
#define	IPOPT_SECUR_EFTO	0x789a
#define	IPOPT_SECUR_MMMM	0xbc4d
#define	IPOPT_SECUR_RESTR	0xaf13
#define	IPOPT_SECUR_SECRET	0xd788
#define	IPOPT_SECUR_TOPSECRET	0x6bc5

/*
 * Internet implementation parameters. 
 */
#define	MAXTTL		255	/* maximum time to live (seconds) */
#define	IPFRAGTTL	60	/* time to live for frags, slowhz */
#define	IPTTLDEC	1	/* subtracted when forwarding */

#define	IP_MSS		576	/* default maximum segment size */
#endif				/* ip_h */
