/* @(#)tcp_var.h	3.8 LAI System V.3 STREAMS TCP/IP  source        */
/*
 * System V STREAMS TCP - Release 3.0 
 *
 * Copyright 1987, 1988, 1989 Lachman Associates, Incorporated (LAI) 
 * All Rights Reserved. 
 *
 * The copyright above and this notice must be preserved in all copies of this
 * source code.  The copyright above does not evidence any actual or intended
 * publication of this source code. 
 *
 * This is unpublished proprietary trade secret source code of Lachman
 * Associates.  This source code may not be copied, disclosed, distributed,
 * demonstrated or licensed except as expressly authorized by Lachman
 * Associates. 
 *
 * System V STREAMS TCP was jointly developed by Lachman Associates and
 * Convergent Technologies. 
 */
/*
 * Copyright (c) 1982, 1986 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this notice is preserved and that due credit is given
 * to the University of California at Berkeley. The name of the University
 * may not be used to endorse or promote products derived from this
 * software without specific prior written permission. This software
 * is provided ``as is'' without express or implied warranty.
 *
 *	@(#)tcp_var.h	7.7 (Berkeley) 2/27/88
 */
#ifndef tcp_var_h
#define tcp_var_h

/*
 * Kernel variables for tcp. 
 */

/*
 * Tcp control block, one per tcp; fields: 
 */
struct tcpcb {
	struct tcpiphdr *seg_next;	/* sequencing queue */
	short           t_state;/* state of this connection */
	short           t_timer[TCPT_NTIMERS];	/* tcp timers */
	short           t_rxtshift;	/* log(2) of rexmt exp. backoff */
	u_short         t_maxseg;	/* maximum segment size */
	short		t_rxtcur;	/* current retransmit value */
	short		t_dupacks;	/* consecutive dup acks recd */
	char            t_force;/* 1 if forcing out a byte */
	u_short         t_flags;
#define	TF_ACKNOW	0x01	/* ack peer immediately */
#define	TF_DELACK	0x02	/* ack, but try to delay it */
#define	TF_NODELAY	0x04	/* don't delay packets to coalesce */
#define	TF_NOOPT	0x08	/* don't use tcp options */
#define	TF_SENTFIN	0x10	/* have sent FIN */
#define TF_IOLOCK	0x20
#define TF_NEEDIN	0x40
#define TF_NEEDOUT	0x80
#define TF_NEEDTIMER	0x0100
	struct tcpiphdr *t_template;	/* skeletal packet for transmit */
	mblk_t         *t_tmplhdr;	/* template back-pointer for dealloc */
	struct inpcb   *t_inpcb;/* back pointer to internet pcb */
	/*
	 * The following fields are used as in the protocol specification.
	 * See RFC783, Dec. 1981, page 21. 
	 */
	/* send sequence variables */
	tcp_seq         snd_una;/* send unacknowledged */
	tcp_seq         snd_nxt;/* send next */
	tcp_seq         snd_up;	/* send urgent pointer */
	tcp_seq         snd_wl1;/* window update seg seq number */
	tcp_seq         snd_wl2;/* window update seg ack number */
	tcp_seq         iss;	/* initial send sequence number */
	u_short         snd_wnd;/* send window */
	/* receive sequence variables */
	u_short         rcv_wnd;/* receive window */
	tcp_seq         rcv_nxt;/* receive next */
	tcp_seq         rcv_up;	/* receive urgent pointer */
	tcp_seq         irs;	/* initial receive sequence number */
	/*
	 * Additional variables for this implementation. 
	 */
	/* receive variables */
	tcp_seq         rcv_adv;/* advertised window */
	/* retransmit variables */
	tcp_seq         snd_max;/* highest sequence number sent used to
				 * recognize retransmits */
	u_short		t_maxwin;	/* max window size to use */
/* congestion control (for slow start, source quench, retransmit after loss) */
	u_short	snd_cwnd;		/* congestion-controlled window */
	u_short snd_ssthresh;		/* snd_cwnd size threshhold for
					 * for slow start exponential to
					 * linear switch */
/*
 * transmit timing stuff.
 * srtt and rttvar are stored as fixed point; for convenience in smoothing,
 * srtt has 3 bits to the right of the binary point, rttvar has 2.
 * "Variance" is actually smoothed difference.
 */
	short           t_idle;	/* inactivity time */
	short           t_rtt;	/* round trip time */
	tcp_seq         t_rtseq;/* sequence number being timed */
	short		t_srtt;		/* smoothed round-trip time */
	short		t_rttvar;	/* variance in round-trip time */
	u_short 	max_rcvd;	/* most peer has sent into window */
	u_short         max_sndwnd;	/* largest window peer has offered */
	/* out-of-band data */
	char            t_oobflags;	/* have some */
	char            t_iobc;	/* input character */
#define	TCPOOB_HAVEDATA	0x01
#define	TCPOOB_HADDATA	0x02
	/* connection queueing */
	struct tcpcb   *t_head;	/* back pointer to accept tcpcb */
	struct tcpcb   *t_q0;	/* queue of partial connections */
	struct tcpcb   *t_q;	/* queue of incoming connections */
	short           t_q0len;/* partials on t_q0 */
	short           t_qlen;	/* number of connections on so_q */
	short           t_qlimit;	/* max number queued connections */
	ushort          t_qsize;/* number of data chars on outq */
	/*
	 * here we save mblks that arrive before the connection is accepted
	 * by the user and those received when the user's queue is full. 
	 */
	mblk_t         *t_qfirst;	/* beginning of queued data */
	mblk_t         *t_qlast;/* end of queued data */
	int             t_iqsize;	/* amount of data on input queue */
	mblk_t	       *t_inq;		/* pending input */
	short           t_linger;	/* linger flag */

	/*
	 * The following stuff is for "one-packet mode", which supports
	 * slow interfaces that can't handle back-to-back packets.
	 * These values are taken from the provider structure of
	 * the interface if IFF_ONEPACKET is on.
	 */
	u_char		t_onepacket;	/* one-packet mode in effect */
	u_short		t_spsize;	/* short packet size */
	int		t_spthresh;	/* short packet threshold */
	int		t_spcount;	/* consecutive short packet count */
};

#define	intotcpcb(ip)	((struct tcpcb *)(ip)->inp_ppcb)
#define qtotcpcb(q) ((struct tcpcb *) ((struct inpcb *) (q)->q_ptr)->inp_ppcb)

/*
 * TCP statistics.
 * Many of these should be kept per connection,
 * but that's inconvenient at the moment.
 */
struct	tcpstat {
	u_long	tcps_connattempt;	/* connections initiated */
	u_long	tcps_accepts;		/* connections accepted */
	u_long	tcps_connects;		/* connections established */
	u_long	tcps_drops;		/* connections dropped */
	u_long	tcps_conndrops;		/* embryonic connections dropped */
	u_long	tcps_closed;		/* conn. closed (includes drops) */
	u_long	tcps_segstimed;		/* segs where we tried to get rtt */
	u_long	tcps_rttupdated;	/* times we succeeded */
	u_long	tcps_delack;		/* delayed acks sent */
	u_long	tcps_timeoutdrop;	/* conn. dropped in rxmt timeout */
	u_long	tcps_rexmttimeo;	/* retransmit timeouts */
	u_long	tcps_persisttimeo;	/* persist timeouts */
	u_long	tcps_keeptimeo;		/* keepalive timeouts */
	u_long	tcps_keepprobe;		/* keepalive probes sent */
	u_long	tcps_keepdrops;		/* connections dropped in keepalive */

	u_long	tcps_sndtotal;		/* total packets sent */
	u_long	tcps_sndpack;		/* data packets sent */
	u_long	tcps_sndbyte;		/* data bytes sent */
	u_long	tcps_sndrexmitpack;	/* data packets retransmitted */
	u_long	tcps_sndrexmitbyte;	/* data bytes retransmitted */
	u_long	tcps_sndacks;		/* ack-only packets sent */
	u_long	tcps_sndprobe;		/* window probes sent */
	u_long	tcps_sndurg;		/* packets sent with URG only */
	u_long	tcps_sndwinup;		/* window update-only packets sent */
	u_long	tcps_sndctrl;		/* control (SYN|FIN|RST) packets sent */

	u_long	tcps_rcvtotal;		/* total packets received */
	u_long	tcps_rcvpack;		/* packets received in sequence */
	u_long	tcps_rcvbyte;		/* bytes received in sequence */
	u_long	tcps_rcvbadsum;		/* packets received with ccksum errs */
	u_long	tcps_rcvbadoff;		/* packets received with bad offset */
	u_long	tcps_rcvshort;		/* packets received too short */
	u_long	tcps_rcvduppack;	/* duplicate-only packets received */
	u_long	tcps_rcvdupbyte;	/* duplicate-only bytes received */
	u_long	tcps_rcvpartduppack;	/* packets with some duplicate data */
	u_long	tcps_rcvpartdupbyte;	/* dup. bytes in part-dup. packets */
	u_long	tcps_rcvoopack;		/* out-of-order packets received */
	u_long	tcps_rcvoobyte;		/* out-of-order bytes received */
	u_long	tcps_rcvpackafterwin;	/* packets with data after window */
	u_long	tcps_rcvbyteafterwin;	/* bytes rcvd after window */
	u_long	tcps_rcvafterclose;	/* packets rcvd after "close" */
	u_long	tcps_rcvwinprobe;	/* rcvd window probe packets */
	u_long	tcps_rcvdupack;		/* rcvd duplicate acks */
	u_long	tcps_rcvacktoomuch;	/* rcvd acks for unsent data */
	u_long	tcps_rcvackpack;	/* rcvd ack packets */
	u_long	tcps_rcvackbyte;	/* bytes acked by rcvd acks */
	u_long	tcps_rcvwinupd;		/* rcvd window update packets */
	u_long	tcps_linger;		/* connections that lingered */
	u_long	tcps_lingerabort;	/* lingers aborted by signal */
	u_long	tcps_lingerexp;		/* linger timer expired */
	u_long	tcps_lingercan;		/* linger timer cancelled */
};

#ifdef TLI_PRIMS
char           *tli_primitives[] = {"CONNECT", "ACCEPT", "DISCONNECT", "DATA",
				 "EX_DATA", "INFORMATION", "BIND", "UNBIND",
				    "UNITDATA", "OPTIONS", "ORDERLY RELEASE",
};
#endif				/* TLI_PRIMS */

#ifdef INKERNEL
extern struct inpcb    tcb;	/* head of queue of active tcpcb's */
extern struct tcpstat  tcpstat;	/* tcp statistics */
struct tcpiphdr *tcp_template();
struct tcpcb   *tcp_close(), *tcp_drop();
struct tcpcb   *tcp_timers(), *tcp_disconnect(), *tcp_usrclosed();
#endif

#endif				/* tcp_var_h */
